@@ -16,15 +16,18 @@
 
 #include "scanner.h"
 
+/* TODO: Could indicate the start of the next page immediately after the last page is received (i.e. before the sane_cancel()) */
 
 enum {
     UPDATE_DEVICES,
     AUTHORIZE,
+    EXPECT_PAGE,
     GOT_PAGE_INFO,
     GOT_LINE,
     SCAN_FAILED,
     PAGE_DONE,
     DOCUMENT_DONE,
+    SCANNING_CHANGED,
     LAST_SIGNAL
 };
 static guint signals[LAST_SIGNAL] = { 0, };
@@ -42,7 +45,8 @@ typedef struct
     gint dpi;
     ScanMode scan_mode;
     gint depth;
-    gboolean multi_page;
+    gboolean type;
+    gint page_width, page_height;
 } ScanJob;
 
 typedef struct
@@ -78,7 +82,7 @@ struct ScannerPrivate
     GAsyncQueue *scan_queue, *authorize_queue;
     GThread *thread;
 
-    ScanDevice *default_device;
+    gchar *default_device;
 
     ScanState state;
     gboolean redetect;
@@ -96,9 +100,10 @@ struct ScannerPrivate
 
     /* Buffer for received line */
     SANE_Byte *buffer;
+    SANE_Int n_used;
 
     SANE_Int bytes_remaining, line_count, pass_number, page_number, notified_page;
-  
+
     gboolean scanning;
 };
 
@@ -108,15 +113,14 @@ G_DEFINE_TYPE (Scanner, scanner, G_TYPE_OBJECT);
 /* Table of scanner objects for each thread (required for authorization callback) */
 static GHashTable *scanners;
 
+#define MAX_READ 10000
+
 
 static gboolean
 send_signal (SignalInfo *info)
 {
     g_signal_emit (info->instance, signals[info->sig], 0, info->data);
 
-    if (info->sig == DOCUMENT_DONE || info->sig == SCAN_FAILED)
-        info->instance->priv->scanning = FALSE;
-
     switch (info->sig) {
     case UPDATE_DEVICES:
         {
@@ -156,6 +160,7 @@ send_signal (SignalInfo *info)
         }
         break;
     default:
+    case EXPECT_PAGE:
     case PAGE_DONE:
     case DOCUMENT_DONE:
     case LAST_SIGNAL:
@@ -182,16 +187,28 @@ emit_signal (Scanner *scanner, guint sig, gpointer data)
 }
 
 
+static void
+set_scanning (Scanner *scanner, gboolean is_scanning)
+{
+    if ((scanner->priv->scanning && !is_scanning) || (!scanner->priv->scanning && is_scanning)) {
+        scanner->priv->scanning = is_scanning;
+        emit_signal (scanner, SCANNING_CHANGED, NULL);
+    }
+}
+
+
 static gint get_device_weight (const gchar *device)
 {
-    /* Use locally connected devices first */
-    if (g_str_has_prefix (device, "usb:"))
-       return 0;
-
+    /* NOTE: This is using trends in the naming of SANE devices, SANE should be able to provide this information better */
+  
     /* Use webcams as a last resort */
     if (g_str_has_prefix (device, "vfl:"))
        return 2;
 
+    /* Use locally connected devices first */
+    if (strstr (device, "usb"))
+       return 0;
+
     return 1;
 }
 
@@ -201,7 +218,7 @@ compare_devices (ScanDevice *device1, ScanDevice *device2)
 {
     gint weight1, weight2;
   
-    /* Should do some fuzzy matching on the last selected device and set that to the default */
+    /* TODO: Should do some fuzzy matching on the last selected device and set that to the default */
 
     weight1 = get_device_weight (device1->name);
     weight2 = get_device_weight (device2->name);
@@ -322,8 +339,7 @@ do_redetect (Scanner *scanner)
     for (device_iter = device_list; *device_iter; device_iter++) {
         const SANE_Device *device = *device_iter;
         ScanDevice *scan_device;
-        GString *label;
-        gchar *c;
+        gchar *c, *label;
 
         g_debug ("Device: name=\"%s\" vendor=\"%s\" model=\"%s\" type=\"%s\"",
                  device->name, device->vendor, device->model, device->type);
@@ -332,20 +348,18 @@ do_redetect (Scanner *scanner)
 
         scan_device->name = g_strdup (device->name);
 
-        label = g_string_new ("");
         /* Abbreviate HP as it is a long string and does not match what is on the physical scanner */
         if (strcmp (device->vendor, "Hewlett-Packard") == 0)
-            g_string_printf (label, "HP %s", device->model);
+            label = g_strdup_printf ("HP %s", device->model);
         else
-            g_string_printf (label, "%s %s", device->vendor, device->model);
+            label = g_strdup_printf ("%s %s", device->vendor, device->model);
         
         /* Replace underscored in name */
-        for (c = label->str; *c; c++)
+        for (c = label; *c; c++)
             if (*c == '_')
                 *c = ' ';
 
-        scan_device->label = label->str;
-        g_string_free (label, FALSE);
+        scan_device->label = label;
 
         devices = g_list_append (devices, scan_device);
     }
@@ -356,8 +370,11 @@ do_redetect (Scanner *scanner)
     scanner->priv->redetect = FALSE;
     scanner->priv->state = STATE_IDLE;
 
-    if (devices)
-        scanner->priv->default_device = g_list_nth_data (devices, 0);
+    g_free (scanner->priv->default_device);
+    if (devices) {
+        ScanDevice *device = g_list_nth_data (devices, 0);
+        scanner->priv->default_device = g_strdup (device->name);
+    }
     else
         scanner->priv->default_device = NULL;
 
@@ -407,16 +424,18 @@ control_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_I
 }
 
 
-static void
+static gboolean
 set_default_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index)
 {
     SANE_Status status;
-    
+
     status = sane_control_option (handle, option_index, SANE_ACTION_SET_AUTO, NULL, NULL);
     g_debug ("sane_control_option (%d, SANE_ACTION_SET_AUTO) -> %s",
              option_index, get_status_string (status));
     if (status != SANE_STATUS_GOOD)
         g_warning ("Error setting default option %s: %s", option->name, sane_strstatus(status));
+
+    return status == SANE_STATUS_GOOD;
 }
 
 
@@ -752,6 +771,16 @@ close_device (Scanner *scanner)
     }
     g_list_free (scanner->priv->job_queue);
     scanner->priv->job_queue = NULL;
+  
+    set_scanning (scanner, FALSE);
+}
+
+static void
+fail_scan (Scanner *scanner, gint error_code, const gchar *error_string)
+{
+    close_device (scanner);
+    scanner->priv->state = STATE_IDLE;
+    emit_signal (scanner, SCAN_FAILED, g_error_new (SCANNER_TYPE, error_code, "%s", error_string));
 }
 
 
@@ -783,15 +812,10 @@ handle_requests (Scanner *scanner)
 
          case REQUEST_START_SCAN:
              scanner->priv->job_queue = g_list_append (scanner->priv->job_queue, request->job);
-             if (g_list_length (scanner->priv->job_queue) == 1)
-                 scanner->priv->state = STATE_OPEN;
              break;
 
          case REQUEST_CANCEL:
-             close_device (scanner);
-             scanner->priv->state = STATE_IDLE;
-             emit_signal (scanner, SCAN_FAILED,
-                          g_error_new (SCANNER_TYPE, SANE_STATUS_CANCELLED, "Scan cancelled - do not report this error"));
+             fail_scan (scanner, SANE_STATUS_CANCELLED, "Scan cancelled - do not report this error");
              break;
 
          case REQUEST_QUIT:
@@ -822,16 +846,13 @@ do_open (Scanner *scanner)
     scanner->priv->option_index = 0;
 
     if (!job->device && scanner->priv->default_device)
-        job->device = g_strdup (scanner->priv->default_device->name);
+        job->device = g_strdup (scanner->priv->default_device);
 
     if (!job->device) {
-        scanner->priv->state = STATE_IDLE;
         g_warning ("No scan device available");
-        scanner->priv->state = STATE_IDLE;
-        emit_signal (scanner, SCAN_FAILED,
-                     g_error_new (SCANNER_TYPE, status,
-                                  /* Error displayed when no scanners to scan with */
-                                  _("No scanners available.  Please connect a scanner.")));
+        fail_scan (scanner, status,
+                   /* Error displayed when no scanners to scan with */
+                   _("No scanners available.  Please connect a scanner."));
         return;
     }
  
@@ -849,19 +870,16 @@ do_open (Scanner *scanner)
   
     g_free (scanner->priv->current_device);
     scanner->priv->current_device = NULL;
-  
+
     status = sane_open (job->device, &scanner->priv->handle);
     g_debug ("sane_open (\"%s\") -> %s", job->device, get_status_string (status));
 
     if (status != SANE_STATUS_GOOD) {
         g_warning ("Unable to get open device: %s", sane_strstatus (status));
         scanner->priv->handle = NULL;
-        close_device (scanner);
-        scanner->priv->state = STATE_IDLE;
-        emit_signal (scanner, SCAN_FAILED,
-                     g_error_new (SCANNER_TYPE, status,
-                                  /* Error displayed when cannot connect to scanner */
-                                  _("Unable to connect to scanner")));
+        fail_scan (scanner, status,
+                   /* Error displayed when cannot connect to scanner */
+                   _("Unable to connect to scanner"));
         return;
     }
 
@@ -914,22 +932,68 @@ do_get_option (Scanner *scanner)
         }
     }
     else if (strcmp (option->name, SANE_NAME_SCAN_SOURCE) == 0) {
+        const char *flatbed_sources[] =
+        {
+            "Flatbed",
+            SANE_I18N ("Flatbed"),
+            "FlatBed",
+            "Normal",
+            SANE_I18N ("Normal"),
+            NULL
+        };
+
         const char *adf_sources[] =
         {
             "Automatic Document Feeder",
             SANE_I18N ("Automatic Document Feeder"),
             "ADF",
-            SANE_I18N ("ADF Front"),
-            "ADF Front",
+            "Automatic Document Feeder(left aligned)", /* Seen in the proprietary brother3 driver */
+            "Automatic Document Feeder(centrally aligned)", /* Seen in the proprietary brother3 driver */
             NULL
         };
 
-        if (job->multi_page) {
-            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))
-                 g_warning ("Unable to set ADF source, please file a bug");
-        }
-        else {
-            set_default_option (scanner->priv->handle, option, option_index);
+        const char *adf_front_sources[] =
+        {
+            "ADF Front",
+            SANE_I18N ("ADF Front"),
+            NULL
+        };
+
+        const char *adf_back_sources[] =
+        {
+            "ADF Back",
+            SANE_I18N ("ADF Back"),
+            NULL
+        };
+
+        const char *adf_duplex_sources[] =
+        {
+            "ADF Duplex",
+            SANE_I18N ("ADF Duplex"),
+            NULL
+        };
+
+        switch (job->type) {
+        case SCAN_SINGLE:
+            if (!set_default_option (scanner->priv->handle, option, option_index))
+                if (!set_constrained_string_option (scanner->priv->handle, option, option_index, flatbed_sources))
+                    g_warning ("Unable to set single page source, please file a bug");
+            break;
+        case SCAN_ADF_FRONT:
+            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_front_sources))
+                if (!!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))                
+                    g_warning ("Unable to set front ADF source, please file a bug");
+            break;
+        case SCAN_ADF_BACK:
+            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_back_sources))
+                if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))
+                    g_warning ("Unable to set back ADF source, please file a bug");
+            break;
+        case SCAN_ADF_BOTH:
+            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_duplex_sources))
+                if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))
+                    g_warning ("Unable to set duplex ADF source, please file a bug");
+            break;
         }
     }
     else if (strcmp (option->name, SANE_NAME_BIT_DEPTH) == 0) {
@@ -942,6 +1006,7 @@ do_get_option (Scanner *scanner)
         {
             SANE_VALUE_SCAN_MODE_COLOR,
             "Color",
+            "24bit Color", /* Seen in the proprietary brother3 driver */
             NULL
         };
         const char *gray_scan_modes[] =
@@ -1016,6 +1081,22 @@ do_get_option (Scanner *scanner)
             break;
         }
     }
+    else if (strcmp (option->name, SANE_NAME_PAGE_WIDTH) == 0) {
+        if (job->page_width > 0.0) {
+            if (option->type == SANE_TYPE_FIXED)
+                set_fixed_option (scanner->priv->handle, option, option_index, job->page_width / 10.0);
+            else
+                set_int_option (scanner->priv->handle, option, option_index, job->page_width / 10);
+        }
+    }
+    else if (strcmp (option->name, SANE_NAME_PAGE_HEIGHT) == 0) {
+        if (job->page_height > 0.0) {
+            if (option->type == SANE_TYPE_FIXED)
+                set_fixed_option (scanner->priv->handle, option, option_index, job->page_height / 10.0);
+            else
+                set_int_option (scanner->priv->handle, option, option_index, job->page_height / 10);
+        }
+    }
 
     /* Test scanner options (hoping will not effect other scanners...) */
     if (strcmp (scanner->priv->current_device, "test") == 0) {
@@ -1060,6 +1141,7 @@ do_complete_document (Scanner *scanner)
     // TODO
 
     emit_signal (scanner, DOCUMENT_DONE, NULL);
+    set_scanning (scanner, FALSE);
 }
 
 
@@ -1067,7 +1149,9 @@ static void
 do_start (Scanner *scanner)
 {
     SANE_Status status;
-  
+
+    emit_signal (scanner, EXPECT_PAGE, NULL);
+
     status = sane_start (scanner->priv->handle);
     g_debug ("sane_start (page=%d, pass=%d) -> %s", scanner->priv->page_number, scanner->priv->pass_number, get_status_string (status));
     if (status == SANE_STATUS_GOOD) {
@@ -1078,12 +1162,9 @@ do_start (Scanner *scanner)
     }
     else {
         g_warning ("Unable to start device: %s", sane_strstatus (status));
-        close_device (scanner);
-        scanner->priv->state = STATE_IDLE;
-        emit_signal (scanner, SCAN_FAILED,
-                     g_error_new (SCANNER_TYPE, status,
-                                  /* Error display when unable to start scan */
-                                  _("Unable to start scan")));
+        fail_scan (scanner, status,
+                   /* Error display when unable to start scan */
+                   _("Unable to start scan"));
     }
 }
 
@@ -1099,12 +1180,9 @@ do_get_parameters (Scanner *scanner)
     g_debug ("sane_get_parameters () -> %s", get_status_string (status));
     if (status != SANE_STATUS_GOOD) {
         g_warning ("Unable to get device parameters: %s", sane_strstatus (status));
-        close_device (scanner);
-        scanner->priv->state = STATE_IDLE;
-        emit_signal (scanner, SCAN_FAILED,
-                     g_error_new (SCANNER_TYPE, status,
-                                  /* Error displayed when communication with scanner broken */
-                                  _("Error communicating with scanner")));
+        fail_scan (scanner, status,
+                   /* Error displayed when communication with scanner broken */
+                   _("Error communicating with scanner"));
         return;
     }
   
@@ -1130,8 +1208,8 @@ do_get_parameters (Scanner *scanner)
     }
 
     /* Prepare for read */
-    scanner->priv->bytes_remaining = scanner->priv->parameters.bytes_per_line;
-    scanner->priv->buffer = g_malloc(sizeof(SANE_Byte) * scanner->priv->bytes_remaining);
+    scanner->priv->buffer = g_malloc(sizeof(SANE_Byte) * MAX_READ);
+    scanner->priv->n_used = 0;
     scanner->priv->line_count = 0;
     scanner->priv->pass_number = 0;
     scanner->priv->state = STATE_READ;
@@ -1142,6 +1220,8 @@ static void
 do_complete_page (Scanner *scanner)
 {
     ScanJob *job;
+
+    emit_signal (scanner, PAGE_DONE, NULL);
   
     job = (ScanJob *) scanner->priv->job_queue->data;
 
@@ -1156,7 +1236,7 @@ do_complete_page (Scanner *scanner)
     }
 
     /* Go back for another page */
-    if (job->multi_page) {
+    if (job->type != SCAN_SINGLE) {
         scanner->priv->page_number++;
         scanner->priv->pass_number = 0;
         emit_signal (scanner, PAGE_DONE, NULL);
@@ -1164,7 +1244,6 @@ do_complete_page (Scanner *scanner)
         return;
     }
 
-    emit_signal (scanner, PAGE_DONE, NULL);  
     do_complete_document (scanner);
 }
 
@@ -1173,13 +1252,15 @@ static void
 do_read (Scanner *scanner)
 {
     SANE_Status status;
-    SANE_Int n_read;
+    SANE_Int n_to_read, n_read;
+
+    /* Read as many bytes as we expect */
+    n_to_read = MAX_READ - scanner->priv->n_used;
 
     status = sane_read (scanner->priv->handle,
-                        scanner->priv->buffer + (scanner->priv->parameters.bytes_per_line - scanner->priv->bytes_remaining),
-                        scanner->priv->bytes_remaining,
-                        &n_read);
-    g_debug ("sane_read (%d) -> (%s, %d)", scanner->priv->bytes_remaining, get_status_string (status), n_read);
+                        scanner->priv->buffer + scanner->priv->n_used,
+                        n_to_read, &n_read);
+    g_debug ("sane_read (%d) -> (%s, %d)", n_to_read, get_status_string (status), n_read);
 
     /* End of variable length frame */
     if (status == SANE_STATUS_EOF &&
@@ -1192,17 +1273,16 @@ do_read (Scanner *scanner)
     /* Communication error */
     if (status != SANE_STATUS_GOOD) {
         g_warning ("Unable to read frame from device: %s", sane_strstatus (status));
-        close_device (scanner);
-        scanner->priv->state = STATE_IDLE;
-        emit_signal (scanner, SCAN_FAILED,
-                     g_error_new (SCANNER_TYPE, status,
-                                  /* Error displayed when communication with scanner broken */
-                                  _("Error communicating with scanner")));
+        fail_scan (scanner, status,
+                   /* Error displayed when communication with scanner broken */
+                   _("Error communicating with scanner"));
         return;
     }
  
-    scanner->priv->bytes_remaining -= n_read;
-    if (scanner->priv->bytes_remaining == 0) {
+    scanner->priv->n_used += n_read;
+  
+    /* Feed out lines */
+    if (scanner->priv->n_used >= scanner->priv->parameters.bytes_per_line) {
         ScanLine *line;
 
         line = g_malloc(sizeof(ScanLine));
@@ -1228,17 +1308,27 @@ do_read (Scanner *scanner)
         line->data = scanner->priv->buffer;
         line->data_length = scanner->priv->parameters.bytes_per_line;
         line->number = scanner->priv->line_count;
-        emit_signal (scanner, GOT_LINE, line);
+        line->n_lines = scanner->priv->n_used / line->data_length;
+
         scanner->priv->buffer = NULL;
+        scanner->priv->line_count += line->n_lines;
 
         /* On last line */
-        scanner->priv->line_count++;
-        if (scanner->priv->parameters.lines > 0 && scanner->priv->line_count == scanner->priv->parameters.lines) {
+        if (scanner->priv->parameters.lines > 0 && scanner->priv->line_count >= scanner->priv->parameters.lines) {
+            emit_signal (scanner, GOT_LINE, line);
             do_complete_page (scanner);
         }
         else {
-            scanner->priv->bytes_remaining = scanner->priv->parameters.bytes_per_line;
-            scanner->priv->buffer = g_malloc(sizeof(SANE_Byte) * scanner->priv->bytes_remaining);
+            int i, n_remaining;
+
+            scanner->priv->buffer = g_malloc(sizeof(SANE_Byte) * MAX_READ);
+            n_remaining = scanner->priv->n_used - (line->n_lines * line->data_length);
+            scanner->priv->n_used = 0;
+            for (i = 0; i < n_remaining; i++) {
+                scanner->priv->buffer[i] = line->data[i + (line->n_lines * line->data_length)];
+                scanner->priv->n_used++;                
+            }
+            emit_signal (scanner, GOT_LINE, line);
         }
     }
 }
@@ -1271,8 +1361,10 @@ scan_thread (Scanner *scanner)
     while (handle_requests (scanner)) {
         switch (scanner->priv->state) {
         case STATE_IDLE:
-             if (scanner->priv->job_queue)
+             if (scanner->priv->job_queue) {
+                 set_scanning (scanner, TRUE);
                  scanner->priv->state = STATE_OPEN;
+             }
              break;
         case STATE_REDETECT:
             do_redetect (scanner);
@@ -1343,22 +1435,41 @@ scanner_is_scanning (Scanner *scanner)
 
 
 void
-scanner_scan (Scanner *scanner, const char *device,
-              gint dpi, ScanMode scan_mode, gint depth, gboolean multi_page)
+scanner_scan (Scanner *scanner, const char *device, ScanOptions *options)
 {
     ScanRequest *request;
+    const gchar *type_string;
+  
+    switch (options->type) {
+    case SCAN_SINGLE:
+        type_string = "SCAN_SINGLE";
+        break;
+    case SCAN_ADF_FRONT:
+        type_string = "SCAN_ADF_FRONT";
+        break;
+    case SCAN_ADF_BACK:
+        type_string = "SCAN_ADF_BACK";
+        break;
+    case SCAN_ADF_BOTH:
+        type_string = "SCAN_ADF_BOTH";
+        break;
+    default:
+        g_assert (FALSE);
+        return;
+    }
 
-    g_debug ("scanner_scan (\"%s\", %d, %s)", device ? device : "(null)", dpi, multi_page ? "TRUE" : "FALSE");
+    g_debug ("scanner_scan (\"%s\", %d, %s)", device ? device : "(null)", options->dpi, type_string);
     request = g_malloc0 (sizeof (ScanRequest));
     request->type = REQUEST_START_SCAN;
     request->job = g_malloc0 (sizeof (ScanJob));
     request->job->device = g_strdup (device);
-    request->job->dpi = dpi;
-    request->job->scan_mode = scan_mode;
-    request->job->depth = depth;
-    request->job->multi_page = multi_page;
+    request->job->dpi = options->dpi;
+    request->job->scan_mode = options->scan_mode;
+    request->job->depth = options->depth;
+    request->job->type = options->type;
+    request->job->page_width = options->paper_width;
+    request->job->page_height = options->paper_height;
     g_async_queue_push (scanner->priv->scan_queue, request);
-    scanner->priv->scanning = TRUE;
 }
 
 
@@ -1413,6 +1524,14 @@ scanner_class_init (ScannerClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__POINTER,
                       G_TYPE_NONE, 1, G_TYPE_POINTER);
+    signals[EXPECT_PAGE] =
+        g_signal_new ("expect-page",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ScannerClass, expect_page),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
     signals[GOT_PAGE_INFO] =
         g_signal_new ("got-page-info",
                       G_TYPE_FROM_CLASS (klass),
@@ -1453,6 +1572,14 @@ scanner_class_init (ScannerClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
+    signals[SCANNING_CHANGED] =
+        g_signal_new ("scanning-changed",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ScannerClass, scanning_changed),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
 
     g_type_class_add_private (klass, sizeof (ScannerPrivate));
 
