@@ -15,6 +15,7 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <math.h>
+#include <unistd.h> // TEMP: Needed for close() in get_temporary_filename()
 
 #include "ui.h"
 #include "book-view.h"
@@ -53,8 +54,8 @@ struct SimpleScanPrivate
     GtkWidget *username_entry, *password_entry;
 
     GtkWidget *preferences_dialog;
-    GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo;
-    GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model;
+    GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo, *page_side_combo, *paper_size_combo;
+    GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model, *page_side_model, *paper_size_model;
     gboolean setting_devices, user_selected_device;
 
     Book *book;
@@ -307,6 +308,52 @@ set_document_hint (SimpleScan *ui, const gchar *document_hint)
 }
 
 
+static void
+set_page_side (SimpleScan *ui, const gchar *document_hint)
+{
+    GtkTreeIter iter;
+
+    if (gtk_tree_model_get_iter_first (ui->priv->page_side_model, &iter)) {
+        do {
+            gchar *d;
+            gboolean have_match;
+
+            gtk_tree_model_get (ui->priv->page_side_model, &iter, 0, &d, -1);
+            have_match = strcmp (d, document_hint) == 0;
+            g_free (d);
+
+            if (have_match) {
+                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->page_side_combo), &iter);                
+                return;
+            }
+        } while (gtk_tree_model_iter_next (ui->priv->page_side_model, &iter));
+     }
+}
+
+
+static void
+set_paper_size (SimpleScan *ui, gint width, gint height)
+{
+    GtkTreeIter iter;
+    gboolean have_iter;
+  
+    for (have_iter = gtk_tree_model_get_iter_first (ui->priv->paper_size_model, &iter);
+         have_iter;
+         have_iter = gtk_tree_model_iter_next (ui->priv->paper_size_model, &iter)) {
+        gint w, h;
+
+        gtk_tree_model_get (ui->priv->paper_size_model, &iter, 0, &w, 1, &h, -1);
+        if (w == width && h == height)
+            break;
+    }
+  
+    if (!have_iter)
+        have_iter = gtk_tree_model_get_iter_first (ui->priv->paper_size_model, &iter);
+    if (have_iter)
+        gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->paper_size_combo), &iter);
+}
+
+
 static gint
 get_text_dpi (SimpleScan *ui)
 {
@@ -333,6 +380,33 @@ get_photo_dpi (SimpleScan *ui)
 }
 
 
+static gchar *
+get_page_side (SimpleScan *ui)
+{
+    GtkTreeIter iter;
+    gchar *mode = NULL;
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->page_side_combo), &iter))
+        gtk_tree_model_get (ui->priv->page_side_model, &iter, 0, &mode, -1);
+    
+    return mode;
+}
+
+
+static gboolean
+get_paper_size (SimpleScan *ui, gint *width, gint *height)
+{
+    GtkTreeIter iter;
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->paper_size_combo), &iter)) {
+        gtk_tree_model_get (ui->priv->paper_size_model, &iter, 0, width, 1, height, -1);
+        return TRUE;
+    }
+  
+    return FALSE;
+}
+
+
 static gchar *
 get_document_hint (SimpleScan *ui)
 {
@@ -346,23 +420,57 @@ get_document_hint (SimpleScan *ui)
 }
 
 
+static ScanOptions *
+get_scan_options (SimpleScan *ui)
+{
+    struct {
+        const gchar *name;
+        ScanMode mode;
+    } profiles[] =
+    {
+        { "text",  SCAN_MODE_LINEART },
+        { "photo", SCAN_MODE_COLOR   },
+        { NULL,    SCAN_MODE_COLOR   }
+    };
+    gint i;
+    gchar *profile_name;
+    ScanOptions *options;
+
+    /* Find this profile */
+    // FIXME: Move this into scan-profile.c
+    profile_name = get_document_hint (ui);
+    for (i = 0; profiles[i].name && strcmp (profiles[i].name, profile_name) != 0; i++);
+  
+    options = g_malloc0 (sizeof (ScanOptions));
+    options->scan_mode = profiles[i].mode;
+    options->depth = 8;
+    if (options->scan_mode == SCAN_MODE_COLOR)
+        options->dpi = get_photo_dpi (ui);
+    else
+        options->dpi = get_text_dpi (ui);
+    get_paper_size (ui, &options->paper_width, &options->paper_height);
+  
+    g_free (profile_name);
+
+    return options;
+}
+
+
 void scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    gchar *device, *mode;
-    gint dpi;
+    gchar *device;
+    ScanOptions *options;
 
     device = get_selected_device (ui);
-    mode = get_document_hint (ui);
-    if (strcmp (mode, "text") == 0) 
-        dpi = get_text_dpi (ui);
-    else
-        dpi = get_photo_dpi (ui);
-    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, FALSE);
+
+    options = get_scan_options (ui);
+    options->type = SCAN_SINGLE;
+    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, options);
     g_free (device);
-    g_free (mode);
+    g_free (options);
 }
 
 
@@ -383,18 +491,23 @@ continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     if (ui->priv->scanning) {
         g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
     } else {
-        gchar *device, *mode;
-        gint dpi;
+        gchar *device, *side;
+        ScanOptions *options;
 
         device = get_selected_device (ui);
-        mode = get_document_hint (ui);
-        if (strcmp (mode, "text") == 0) 
-            dpi = get_text_dpi (ui);
+        options = get_scan_options (ui);
+        side = get_page_side (ui);
+        if (strcmp (side, "front") == 0)
+            options->type = SCAN_ADF_FRONT;
+        else if (strcmp (side, "back") == 0)
+            options->type = SCAN_ADF_BACK;
         else
-            dpi = get_photo_dpi (ui);
-        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, TRUE);
+            options->type = SCAN_ADF_BOTH;
+
+        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, options);
         g_free (device);
-        g_free (mode);
+        g_free (side);
+        g_free (options);
     }
 }
 
@@ -439,7 +552,7 @@ page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
     if (page_has_crop (page)) {
         char *crop_name;
       
-        // FIXME: Make more generic
+        // FIXME: Make more generic, move into page-size.c and reuse
         crop_name = page_get_named_crop (page);
         if (crop_name) {
             if (strcmp (crop_name, "A4") == 0)
@@ -463,11 +576,68 @@ page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
         name = "no_crop_menuitem";
 
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, name)), TRUE);
+    gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (gtk_builder_get_object (ui->priv->builder, "crop_toolbutton")), page_has_crop (page));
 
     ui->priv->updating_page_menu = FALSE;
 }
 
 
+// FIXME: Duplicated from simple-scan.c
+static gchar *
+get_temporary_filename (const gchar *prefix, const gchar *extension)
+{
+    gint fd;
+    gchar *filename, *path;
+    GError *error = NULL;
+
+    /* NOTE: I'm not sure if this is a 100% safe strategy to use g_file_open_tmp(), close and
+     * use the filename but it appears to work in practise */
+
+    filename = g_strdup_printf ("%s-XXXXXX.%s", prefix, extension);
+    fd = g_file_open_tmp (filename, &path, &error);
+    g_free (filename);
+    if (fd < 0) {
+        g_warning ("Error saving page for viewing: %s", error->message);
+        g_clear_error (&error);
+        return NULL;
+    }
+    close (fd);
+
+    return path;
+}
+
+
+static void
+show_page_cb (BookView *view, Page *page, SimpleScan *ui)
+{
+    gchar *path, *uri;
+    GdkScreen *screen;
+    GError *error = NULL;
+  
+    path = get_temporary_filename ("scanned-page", "tiff");
+    if (!path)
+        return;
+
+    screen = gtk_widget_get_screen (GTK_WIDGET (ui->priv->window));
+
+    uri = g_filename_to_uri (path, NULL, NULL);
+    if (page_save_tiff (page, uri, &error))
+        gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+
+    if (error) {
+        ui_show_error (ui,
+                       /* Error message display when unable to preview image */
+                       _("Unable to open image preview application"),
+                       error->message,
+                       FALSE);
+        g_clear_error (&error);
+    }
+
+    g_free (uri);
+    g_free (path);
+}
+
+
 void rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
@@ -552,12 +722,18 @@ custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
         set_crop (ui, "custom");
 }
 
-void crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+void crop_toolbutton_toggled_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
-crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+crop_toolbutton_toggled_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, "custom_crop_menuitem")), TRUE);
+    if (ui->priv->updating_page_menu)
+        return;
+  
+    if (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (widget)))
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, "custom_crop_menuitem")), TRUE);
+    else
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, "no_crop_menuitem")), TRUE);
 }
 
 
@@ -705,7 +881,7 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     };
 
     /* Get directory to save to */
-    directory = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/save_directory", NULL);
+    directory = gconf_client_get_string (ui->priv->client, GCONF_DIR "/save_directory", NULL);
     if (!directory || directory[0] == '\0') {
         g_free (directory);
         directory = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS));
@@ -789,7 +965,7 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         g_free (uri);
     }
 
-    gconf_client_set_string (ui->priv->client, "/apps/simple-scan/save_directory",
+    gconf_client_set_string (ui->priv->client, GCONF_DIR "/save_directory",
                              gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog)),
                              NULL);
   
@@ -875,22 +1051,14 @@ help_contents_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
     screen = gtk_widget_get_screen (GTK_WIDGET (ui->priv->window));
     gtk_show_uri (screen, "ghelp:simple-scan", gtk_get_current_event_time (), &error);
 
-    if (error != NULL)
+    if (error)
     {
-        GtkWidget *d;
-        /* Error message displayed when unable to launch help browser */
-        const char *message = _("Unable to open help file");
-
-        d = gtk_message_dialog_new (GTK_WINDOW (ui->priv->window),
-                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                    "%s", message);
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (d),
-                                                  "%s", error->message);
-        g_signal_connect (d, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-        gtk_window_present (GTK_WINDOW (d));
-
-        g_error_free (error);
+        ui_show_error (ui,
+                       /* Error message displayed when unable to launch help browser */
+                       _("Unable to open help file"),
+                       error->message,
+                      FALSE);
+        g_clear_error (&error);
     }
 }
 
@@ -942,30 +1110,37 @@ static void
 quit (SimpleScan *ui)
 {
     char *device, *document_type;
+    gint paper_width = 0, paper_height = 0;
     gint i;
 
+    // FIXME: Warn if document with unsaved changes
+
     device = get_selected_device (ui);
     if (device) {
-        gconf_client_set_string(ui->priv->client, "/apps/simple-scan/selected_device", device, NULL);
+        gconf_client_set_string(ui->priv->client, GCONF_DIR "/selected_device", device, NULL);
         g_free (device);
     }
 
     document_type = get_document_hint (ui);
-    gconf_client_set_string(ui->priv->client, "/apps/simple-scan/document_type", document_type, NULL);
+    gconf_client_set_string(ui->priv->client, GCONF_DIR "/document_type", document_type, NULL);
     g_free (document_type);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/text_dpi", get_text_dpi (ui), NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/photo_dpi", get_photo_dpi (ui), NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/text_dpi", get_text_dpi (ui), NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/photo_dpi", get_photo_dpi (ui), NULL);
+    gconf_client_set_string (ui->priv->client, GCONF_DIR "/page_side", get_page_side (ui), NULL);
+    get_paper_size (ui, &paper_width, &paper_height);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/paper_width", paper_width, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/paper_height", paper_height, NULL);
 
-    gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_width", ui->priv->window_width, NULL);
-    gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_height", ui->priv->window_height, NULL);
-    gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/window_is_maximized", ui->priv->window_is_maximized, NULL);
+    gconf_client_set_int(ui->priv->client, GCONF_DIR "/window_width", ui->priv->window_width, NULL);
+    gconf_client_set_int(ui->priv->client, GCONF_DIR "/window_height", ui->priv->window_height, NULL);
+    gconf_client_set_bool(ui->priv->client, GCONF_DIR "/window_is_maximized", ui->priv->window_is_maximized, NULL);
 
     for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_page_orientation; i++);
     if (orientation_keys[i].key != NULL)
-        gconf_client_set_string(ui->priv->client, "/apps/simple-scan/scan_direction", orientation_keys[i].key, NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_width", ui->priv->default_page_width, NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_height", ui->priv->default_page_height, NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_dpi", ui->priv->default_page_dpi, NULL);
+        gconf_client_set_string(ui->priv->client, GCONF_DIR "/scan_direction", orientation_keys[i].key, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/page_width", ui->priv->default_page_width, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/page_height", ui->priv->default_page_height, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/page_dpi", ui->priv->default_page_dpi, NULL);
    
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
 }
@@ -1112,8 +1287,12 @@ ui_load (SimpleScan *ui)
     GtkBuilder *builder;
     GError *error = NULL;
     GtkCellRenderer *renderer;
-    gchar *device, *document_type, *scan_direction;
-    gint dpi;
+    gchar *device, *document_type, *scan_direction, *page_side;
+    gint dpi, paper_width, paper_height;
+
+    gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), ICON_DIR);
+
+    gtk_window_set_default_icon_name ("scanner");
 
     builder = ui->priv->builder = gtk_builder_new ();
     gtk_builder_add_from_file (builder, UI_DIR "simple-scan.ui", &error);
@@ -1157,12 +1336,32 @@ ui_load (SimpleScan *ui)
     ui->priv->text_dpi_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->text_dpi_combo));
     ui->priv->photo_dpi_combo = GTK_WIDGET (gtk_builder_get_object (builder, "photo_dpi_combo"));
     ui->priv->photo_dpi_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->photo_dpi_combo));
+    ui->priv->page_side_combo = GTK_WIDGET (gtk_builder_get_object (builder, "page_side_combo"));
+    ui->priv->page_side_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->page_side_combo));
+    ui->priv->paper_size_combo = GTK_WIDGET (gtk_builder_get_object (builder, "paper_size_combo"));
+    ui->priv->paper_size_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->paper_size_combo));
 
-    dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/text_dpi", NULL);
+    GtkTreeIter iter;
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 0, 1, 0, 2, "Automatic", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 2050, 1, 1480, 2, "A6", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 1480, 1, 2100, 2, "A5", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 2100, 1, 1970, 2, "A4", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 2159, 1, 2794, 2, "Letter", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 2159, 1, 3556, 2, "Legal", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 1016, 1, 1524, 2, "4×6", -1);
+
+    dpi = gconf_client_get_int (ui->priv->client, GCONF_DIR "/text_dpi", NULL);
     if (dpi <= 0)
         dpi = DEFAULT_TEXT_DPI;
     set_dpi_combo (ui->priv->text_dpi_combo, DEFAULT_TEXT_DPI, dpi);
-    dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/photo_dpi", NULL);
+    dpi = gconf_client_get_int (ui->priv->client, GCONF_DIR "/photo_dpi", NULL);
     if (dpi <= 0)
         dpi = DEFAULT_PHOTO_DPI;
     set_dpi_combo (ui->priv->photo_dpi_combo, DEFAULT_PHOTO_DPI, dpi);
@@ -1171,15 +1370,31 @@ ui_load (SimpleScan *ui)
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, "text", 1);
 
-    device = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/selected_device", NULL);
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, "text", 1);
+    page_side = gconf_client_get_string (ui->priv->client, GCONF_DIR "/page_side", NULL);
+    if (page_side) {
+        set_page_side (ui, page_side);
+        g_free (page_side);
+    }
+
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->paper_size_combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->paper_size_combo), renderer, "text", 2);
+    paper_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/paper_width", NULL);
+    paper_height = gconf_client_get_int (ui->priv->client, GCONF_DIR "/paper_height", NULL);
+    set_paper_size (ui, paper_width, paper_height);
+
+    device = gconf_client_get_string (ui->priv->client, GCONF_DIR "/selected_device", NULL);
     if (device) {
         GtkTreeIter iter;
         if (find_scan_device (ui, device, &iter))
             gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
         g_free (device);
     }
-    
-    document_type = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/document_type", NULL);
+
+    document_type = gconf_client_get_string (ui->priv->client, GCONF_DIR "/document_type", NULL);
     if (document_type) {
         set_document_hint (ui, document_type);
         g_free (document_type);
@@ -1187,6 +1402,7 @@ ui_load (SimpleScan *ui)
 
     ui->priv->book_view = book_view_new ();
     g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
+    g_signal_connect (ui->priv->book_view, "show-page", G_CALLBACK (show_page_cb), ui);
     book_view_set_widgets (ui->priv->book_view,
                            ui->priv->preview_box,
                            ui->priv->preview_area,
@@ -1194,7 +1410,7 @@ ui_load (SimpleScan *ui)
                            GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
 
     /* Find default page details */
-    scan_direction = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/scan_direction", NULL);
+    scan_direction = gconf_client_get_string(ui->priv->client, GCONF_DIR "/scan_direction", NULL);
     ui->priv->default_page_orientation = TOP_TO_BOTTOM;
     if (scan_direction) {
         gint i;
@@ -1202,26 +1418,26 @@ ui_load (SimpleScan *ui)
         if (orientation_keys[i].key != NULL)
             ui->priv->default_page_orientation = orientation_keys[i].orientation;
     }
-    ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_width", NULL);
+    ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_width", NULL);
     if (ui->priv->default_page_width <= 0)
         ui->priv->default_page_width = 595;
-    ui->priv->default_page_height = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_height", NULL);
+    ui->priv->default_page_height = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_height", NULL);
     if (ui->priv->default_page_height <= 0)
         ui->priv->default_page_height = 842;
-    ui->priv->default_page_dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_dpi", NULL);
+    ui->priv->default_page_dpi = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_dpi", NULL);
     if (ui->priv->default_page_dpi <= 0)
         ui->priv->default_page_dpi = 72;
 
     /* Restore window size */
-    ui->priv->window_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_width", NULL);
+    ui->priv->window_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/window_width", NULL);
     if (ui->priv->window_width <= 0)
         ui->priv->window_width = 600;
-    ui->priv->window_height = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_height", NULL);
+    ui->priv->window_height = gconf_client_get_int (ui->priv->client, GCONF_DIR "/window_height", NULL);
     if (ui->priv->window_height <= 0)
         ui->priv->window_height = 400;
     g_debug ("Restoring window to %dx%d pixels", ui->priv->window_width, ui->priv->window_height);
     gtk_window_set_default_size (GTK_WINDOW (ui->priv->window), ui->priv->window_width, ui->priv->window_height);
-    ui->priv->window_is_maximized = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/window_is_maximized", NULL);
+    ui->priv->window_is_maximized = gconf_client_get_bool (ui->priv->client, GCONF_DIR "/window_is_maximized", NULL);
     if (ui->priv->window_is_maximized) {
         g_debug ("Restoring window to maximized");
         gtk_window_maximize (GTK_WINDOW (ui->priv->window));
@@ -1308,24 +1524,22 @@ ui_start (SimpleScan *ui)
 
 /* Generated with glib-genmarshal */
 static void
-g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN (GClosure     *closure,
-                                                         GValue       *return_value G_GNUC_UNUSED,
-                                                         guint         n_param_values,
-                                                         const GValue *param_values,
-                                                         gpointer      invocation_hint G_GNUC_UNUSED,
-                                                         gpointer      marshal_data)
+g_cclosure_user_marshal_VOID__STRING_POINTER (GClosure     *closure,
+                                              GValue       *return_value G_GNUC_UNUSED,
+                                              guint         n_param_values,
+                                              const GValue *param_values,
+                                              gpointer      invocation_hint G_GNUC_UNUSED,
+                                              gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN) (gpointer       data1,
-                                                                gconstpointer  arg_1,
-                                                                gint           arg_2,
-                                                                gconstpointer  arg_3,
-                                                                gboolean       arg_4,
-                                                                gpointer       data2);
-  register GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN callback;
+  typedef void (*GMarshalFunc_VOID__STRING_POINTER) (gpointer       data1,
+                                                     gconstpointer  arg_1,
+                                                     gconstpointer  arg_2,
+                                                     gpointer       data2);
+  register GMarshalFunc_VOID__STRING_POINTER callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 5);
+  g_return_if_fail (n_param_values == 3);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -1337,13 +1551,11 @@ g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__STRING_POINTER) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_value_get_string (param_values + 1),
-            g_value_get_int (param_values + 2),
-            g_value_get_string (param_values + 3),
-            g_value_get_boolean (param_values + 4),
+            g_value_get_pointer (param_values + 2),
             data2);
 }
 
@@ -1357,8 +1569,8 @@ ui_class_init (SimpleScanClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, start_scan),
                       NULL, NULL,
-                      g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN,
-                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_BOOLEAN);
+                      g_cclosure_user_marshal_VOID__STRING_POINTER,
+                      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
     signals[STOP_SCAN] =
         g_signal_new ("stop-scan",
                       G_TYPE_FROM_CLASS (klass),
@@ -1406,7 +1618,7 @@ ui_init (SimpleScan *ui)
     g_signal_connect (ui->priv->book, "page-added", G_CALLBACK (page_added_cb), ui);
    
     ui->priv->client = gconf_client_get_default();
-    gconf_client_add_dir(ui->priv->client, "/apps/simple-scan", GCONF_CLIENT_PRELOAD_NONE, NULL);
+    gconf_client_add_dir(ui->priv->client, GCONF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
 
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
